@@ -1,8 +1,10 @@
-import itertools
 from threading import Lock
 from concurrent.futures import Future, CancelledError, TimeoutError
 
 
+_UNDEFINED = object()
+
+
 class ClosedError(Exception):
     pass
 
@@ -38,7 +40,10 @@ class _Chain(object):
 
 
 class _SuccessionIterator(object):
-    def __init__(self, head, timeout=None):
+    def __init__(self, head, prelude=None, timeout=None):
+        if prelude is None:
+            prelude = []
+        self._prelude = iter(prelude)
         self._next = head
         self._timeout = timeout
 
@@ -46,6 +51,12 @@ class _SuccessionIterator(object):
         return self
 
     def __next__(self):
+        try:
+            return next(self._prelude)
+        except StopIteration:
+            # prelude empty, yield from chain
+            pass
+
         try:
             result, self._next = self._next.wait(self._timeout)
             return result
@@ -54,8 +65,9 @@ class _SuccessionIterator(object):
 
 
 class Succession(object):
-    def __init__(self, initial=None, compress=None):
+    def __init__(self, initial=None, drop=False, compress=None):
         self._lock = Lock()
+        self._drop_after_push = drop
         self._compress_function = compress
 
         self._prelude = []
@@ -71,7 +83,7 @@ class Succession(object):
             raise StopIteration()
 
     def head(self):
-        """Returns an non-blocking iterator over all items that have already
+        """Returns a non-blocking iterator over all items that have already
         been added to the succession.
         """
         with self._lock:
@@ -81,8 +93,8 @@ class Succession(object):
         """Returns an iterator over items in the succession without acquiring
         the succession lock.
         """
-        return itertools.chain(
-            self._prelude, _SuccessionIterator(self._root, timeout)
+        return _SuccessionIterator(
+            self._root, prelude=self._prelude, timeout=timeout
         )
 
     def iter(self, timeout=None):
@@ -100,11 +112,16 @@ class Succession(object):
     def __iter__(self):
         return self.iter()
 
-    def push(self, value):
+    def push(self, value, *, drop=_UNDEFINED, compress=_UNDEFINED):
+        drop = self._drop_after_push if drop is _UNDEFINED else drop
+        drop = self._compress_function if compress is _UNDEFINED else compress
+
         with self._lock:
             self._cursor = self._cursor.push(value)
-            if self._compress_function is not None:
-                self._prelude = self._compress_function(self._prelude, value)
+            if self._drop_after_push:
+                self._root = self._cursor
+            elif self._compress_function is not None:
+                self._prelude = self._compress_function(self._head())
                 self._root = self._cursor
 
     def close(self):
@@ -113,9 +130,26 @@ class Succession(object):
         with self._lock:
             self._cursor.close()
 
+    def compress(self, function):
+        """Applies function to an iterable of items in the succession and
+        replaces the items with the result.
+
+        :param function:
+            A function taking an iterable of items and returning an equivalent
+            iterable of items to replace them with
+        """
+        with self._lock:
+            self._prelude = list(function(self._head()))
+            self._root = self._cursor
+
     def drop(self):
+        """Remove all items from the succession
+
+        This does not affect existing iterators over the succession.
+        """
         with self._lock:
             dropped = self._head()
+            self._prelude = []
             self._root = self._cursor
             return dropped
 
