@@ -61,11 +61,21 @@ class TestSuccession(unittest.TestCase):
 
         self.assertEqual(list(succession), [1, 2, 3, 4, 5])
 
-    def test_compress(self):
-        def accumulate(start, new):
-            return [sum(start) + new]
+    def test_release_iter(self):
+        succession = Succession()
+        root = weakref.ref(succession._root)
+        iterator = weakref.ref(iter(succession))
 
-        succession = Succession(compress=accumulate)
+        for i in [1, 2, 3, 4, 5]:
+            succession.push(i)
+        succession.drop()
+
+        gc.collect()
+        self.assertIsNone(root())
+        self.assertIsNone(iterator())
+
+    def test_compress(self):
+        succession = Succession(compress=lambda items: [sum(items)])
 
         from_start = succession.iter()
 
@@ -92,10 +102,13 @@ class TestSuccession(unittest.TestCase):
 
     def test_drop(self):
         succession = Succession()
+        root = weakref.ref(succession._root)
 
         for i in [1, 2, 3, 4, 5]:
             succession.push(i)
         succession.drop()
+        gc.collect()
+        self.assertIsNone(root())
 
         for i in [6, 7, 8, 9, 10]:
             succession.push(i)
